# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is the Bugzilla HTTP REST API proxy.
#
# The Initial Developer of the Original Code is
# the Mozilla Foundation.
# Portions created by the Initial Developer are Copyright (C) 2009
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   The Original Software was written to the Glory of God by 
#   Gervase Markham <written.to.the.glory.of.God@gerv.net>.
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

package Bugzilla::API::Controller::Bug;

use strict;
use warnings;
use parent 'Bugzilla::API::Controller::REST';

use Bugzilla::API::Controller::Utils;

sub history :LocalRegex('^(\d+)/history$') :ActionClass('CrossSiteREST') { }
sub cc      :LocalRegex('^(\d+)/cc$')      :ActionClass('CrossSiteREST') { }
sub flag    :LocalRegex('^(\d+)/flag$')    :ActionClass('CrossSiteREST') { }
sub related :LocalRegex('^(\d+)/related$') :ActionClass('CrossSiteREST') { }
sub comment :LocalRegex('^(\d+)/comment$') :ActionClass('CrossSiteREST') { }
sub attachment :LocalRegex('^(\d+)/attachment$') 
                                           :ActionClass('CrossSiteREST') { }
sub one_bug :LocalRegex('^(.+)$')          :ActionClass('CrossSiteREST') { }
sub bug     :Path                          :ActionClass('CrossSiteREST') { }

##############################################################################
# Bug list
##############################################################################
sub bug_GET {
  my ($self, $c) = @_;

  # We pass through all query parameters from the client to the search
  my $query = $c->req->params;
  
  my $result = Bugzilla::API::Model::Bug->list($c, $query);
  
  ok_or_error($self, $c, $result);
}

##############################################################################
# Get single bug
##############################################################################
sub one_bug_GET {
  my ($self, $c) = @_;

  my $bugid = $c->req->captures->[0];
  
  my $result = Bugzilla::API::Model::Bug->get($c, $bugid);
  
  include_exclude_fields($c, $result, ["comments", "history", 
                                       "attachments.data",
                                       "attachments.encoding"]);
  
  ok_or_error($self, $c, $result);
}

##############################################################################
# Update single bug
##############################################################################
sub one_bug_PUT {
  my ($self, $c) = @_;
  
  my $bugid = $c->req->captures->[0];

  my $result;
  
  if ($c->req->data) {
    $result = Bugzilla::API::Model::Bug->update($c, $bugid, $c->req->data);
  }
  else {
    $result = { error => 1, 'message' => "No valid data supplied for update" };
  }
  
  accepted_or_error($self, $c, $result);  
}

##############################################################################
# Add a bug
##############################################################################
sub bug_POST {
  my ($self, $c) = @_;

  my $result;
  
  if ($c->req->data) {
    $result = Bugzilla::API::Model::Bug->create($c, $c->req->data);
  }
  else {
    $result = { error => 1, 'message' => "No data supplied for create" };
  }
  
  created_or_error($self, $c, $result);
}

##############################################################################
# History
##############################################################################
sub history_GET {
  my ($self, $c) = @_;
  
  my $bugid = $c->req->captures->[0];
  
  my $result = Bugzilla::API::Model::Bug->history($c, $bugid);
  
  # Reduce down to single 'history' hash key
  $result = $result->{'error'} ? 
      $result : 
      { 'history' => $result->{'bugs'}->[0]->{'history'} };
  
  include_exclude_fields($c, $result->{'history'});
  
  ok_or_error($self, $c, $result);
}

##############################################################################
# Comments
##############################################################################
sub comment_GET {
  my ($self, $c) = @_;
  
  my $bugid = $c->req->captures->[0];
  my $new_since = $c->req->params->{'new_since'};
  
  my $result = Bugzilla::API::Model::Bug->comments($c, $bugid, $new_since);

  include_exclude_fields($c, $result->{'comments'});
  
  ok_or_error($self, $c, $result);
}

##############################################################################
# Add a comment
##############################################################################
sub comment_POST {
  my ($self, $c) = @_;
  
  my $bugid = $c->req->captures->[0];
  
  my $result;
  
  if ($c->req->data) {
    $result = Bugzilla::API::Model::Bug->add_comment($c, 
                                                     $bugid, 
                                                     $c->req->data);
  }
  else {
    $result = { error => 1, 'message' => "No data supplied for comment" }
  }
  
  created_or_error($self, $c, $result); 
}

##############################################################################
# Attachments
##############################################################################
sub attachment_GET {
  my ($self, $c) = @_;
  
  my $bugid = $c->req->captures->[0];
  
  # Note 'list' call is in Model::Bug
  my $result = Bugzilla::API::Model::Bug->attachments($c, $bugid);

  include_exclude_fields($c, $result->{'attachments'}, ["data", "encoding"]);
  
  ok_or_error($self, $c, $result);
}

##############################################################################
# Add an attachment
##############################################################################
sub attachment_POST {
  my ($self, $c) = @_;

  my $result;
  
  my $bugid = $c->req->captures->[0];
  
  if ($c->req->data) {
    $result = Bugzilla::API::Model::Attachment->create($c, 
                                                       $bugid, 
                                                       $c->req->data);
  }
  else {
    $result = { error => 1, 'message' => "No data supplied for attachment" };
  }
  
  created_or_error($self, $c, $result);
}

##############################################################################
# Flags
##############################################################################
sub flag_GET {
  my ($self, $c) = @_;
  
  my $bugid = $c->req->captures->[0];
  
  my $result = Bugzilla::API::Model::Bug->flags($c, $bugid);

  include_exclude_fields($c, $result->{'flags'});
  
  ok_or_error($self, $c, $result);
}

1;
