# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is the Bugzilla HTTP REST API proxy.
#
# The Initial Developer of the Original Code is
# the Mozilla Foundation.
# Portions created by the Initial Developer are Copyright (C) 2009
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   The Original Software was written to the Glory of God by 
#   Gervase Markham <written.to.the.glory.of.God@gerv.net>.
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

package Bugzilla::API::Model::Count;

use strict;
use warnings;
use parent 'Catalyst::Model';

use Text::CSV_XS;
use LWP::UserAgent;
use Data::Dumper;
use URI;
use URI::QueryParam;
use Bugzilla::API::Model::Utils;
use Bugzilla::API::Request;

##############################################################################
# Tables of bug counts
##############################################################################
sub get {
  my ($self, $c, $query) = @_;

  my $url = new URI($c->config->{'bugzilla_url'} . 'report.cgi');
  $url->query_form($query);
  
  downgrade_search_url($c, $url);
  
  my $dimensions = $url->query_param("x_axis_field") ?
                   $url->query_param("y_axis_field") ?
                   $url->query_param("z_axis_field") ? 3 : 2 : 1 : 0;
                   
  if ($dimensions == 0) {
    # Bugzilla forces us to specify at least an x_axis_field, so we specify
    # a bogus one and fix the data when we get it back
    $url->query_param("x_axis_field", "bug_status");
  }
  
  $url->query_param('ctype', "csv");  
  $url->query_param("action", "wrap");
  $url->query_param("format", "table");
  
  my $res = make_request($c, "GET", $url);

  my $result;

  if ($res->is_success && $res->decoded_content !~ /^<!DOCTYPE/) {
    # Parse CSV
    my $csv = Text::CSV_XS->new({ binary => 1, 
                                  allow_loose_quotes => 1,
                                  allow_loose_escapes => 1 });

    # Convert string $res->decoded_content to filehandle $fh
    open(my $fh, "<:encoding(UTF-8)", \$res->decoded_content);
    
    my (@x_labels, @y_labels, @z_labels);
    my @data;
    
    while (!$fh->eof()) {
      @x_labels = @{ $csv->getline($fh) };
      @y_labels = ();
      my @table = ();
      
      # Extract z label
      my $cornercell = shift(@x_labels);
      $cornercell =~ /"[^"]+"([^"]+)"/;
      push(@z_labels, $1 || $cornercell);
      
      while (my $line = $csv->getline($fh)) {
        # There is a blank line between each table in 3d
        if (!$line->[0]) {
          last;
        }

        push (@y_labels, shift(@{ $line }));
        
        # Last value can have a space, and therefore quotes (bug in template?)
        $line->[-1] =~ s/["\s]//g;
        
        # Numify all data, so it encodes as number and not string
        @{ $line } = map { $_ += 0 } @{ $line };
        
        push (@table, $#{$line} ? $line : $line->[0]);
      }
      
      push(@data, \@table);
    }      
    
    if    ($dimensions == 0) {
      my $sum = 0;
      
      # If the search returns no results, we just get an 0-byte file back
      # and so there is no data at all.
      if (@data) {
        $sum += $_ for @{ pop(@data) };
      }
      
      $result = { 
        'data' => $sum 
      };
    }
    elsif ($dimensions == 1) {
      $result = {
        # Note this mix of x and y is correct; 1-dimensional tables arrive 
        # the wrong way round
        'x_labels' => \@y_labels,
        'data' => pop(@data) || []
      };
    }
    elsif ($dimensions == 2) {
      $result = {
        'x_labels' => \@x_labels,
        'y_labels' => \@y_labels,
        'data' => pop(@data) || [[]]
      };
    }
    elsif ($dimensions == 3) {
      if (@data > 1 && $z_labels[-1] eq "-total-") {
        # Last table is a total, which we discard 
        pop(@data);
        pop(@z_labels);
      }
      
      $result = {
        'x_labels' => \@x_labels,
        'y_labels' => \@y_labels,
        'z_labels' => \@z_labels,
        'data' => @data ? \@data : [[[]]]
      };
    }
  }
  else {
    $result = http_result2error($res);
  }  
    
  return $result;
}

1;
