# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is the Bugzilla HTTP REST API proxy.
#
# The Initial Developer of the Original Code is
# the Mozilla Foundation.
# Portions created by the Initial Developer are Copyright (C) 2009
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   The Original Software was written to the Glory of God by 
#   Gervase Markham <written.to.the.glory.of.God@gerv.net>.
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

# Package to manage requests to Bugzilla.

package Bugzilla::API::Request;

use strict;
use warnings;
use base 'Exporter';

use vars qw(@ISA @EXPORT @EXPORT_OK);

@EXPORT = qw(
  make_request
  make_rpc_request
  submit_form
);

use Data::Dumper;
use Bugzilla::API::Model::Utils;
use Time::HiRes qw(gettimeofday tv_interval);
use HTTP::Request::Common;

# post_params is a hash reference
sub make_request {
  my ($c, $method, $url, $post_params) = @_;

  my $ua = LWP::UserAgent->new;

  # Auth
  do_auth($c, $url, $ua);

  my $response;
  
  if ($method eq "GET") {
    $c->log->info(stringify($c->req->uri) . " GET => " . stringify($url));
    $c->stats->profile(begin => "Bugzilla GET request: " . $url->path);
    $response = $ua->get($url);
    $c->stats->profile(end => "Bugzilla GET request: " . $url->path);
  }
  else { # POST
    # Set up form encodings
    $c->log->info(stringify_req($c->req) . " POST => " . stringify($url) .
                  " | " . stringify_hashref($post_params));
    
    # This function can't be used for multipart/form-data file upload
    # as ->post() defaults to using application/x-www-form-urlencoded.
    # See submit_form instead.
    $c->stats->profile(begin => "Bugzilla POST request: " . $url->path);
    $response = $ua->post($url, $post_params);
    $c->stats->profile(end => "Bugzilla POST request: " . $url->path);
  }
  
  return $response;
}

# See docs for HTTP::Request::Common for details on how to fake form submit
sub submit_form {
  my ($c, $url, $content) = @_;
  
  my $ua = LWP::UserAgent->new;
  
  # Auth
  do_auth($c, $url, $ua);

  my $req = POST $url, Content_Type => 'form-data', Content => [%$content];

  $c->log->info(stringify_req($c->req) . " => " . stringify($url));  

  $c->stats->profile(begin => "Bugzilla POST request: " . $url->path);
  my $response = $ua->request($req);
  $c->stats->profile(end => "Bugzilla POST request: " . $url->path);
  
  return $response;
}

my %clients;

sub make_rpc_request {
  my ($c, $method, $payload) = @_;
  my $client;
  my $result = {};

  my $url = new URI($c->config->{'bugzilla_url'} . 'xmlrpc.cgi');
  
  if ($c->req->params->{'cookie'}) 
  {
    my $cookie_jar = do_auth($c, $url);
    $client = BZ::Client->new("url" => $url);
    # Hack the client so that it uses the cookies we've been passed
    $client->xmlrpc()->user_agent()->cookie_jar($cookie_jar);
  }
  else {
    if ($c->config->{'bugzilla_version'} >= 3.6) {
      # 3.6 and above support direct XML-RPC login on the request
      if ($c->req->params->{'password'}) {
        $payload->{'Bugzilla_login'} = $c->req->params->{'username'};
        $payload->{'Bugzilla_password'} = $c->req->params->{'password'};
      }
      $client = BZ::Client->new("url" => $url);
    }
    else {      
      # Have to log in manually; cache client object
      # If no username and password are passed, we use a "public" client
      # which is stored with the hash key of "|".
      my $username = $c->req->params->{'username'} || "";
      my $password = $c->req->params->{'password'} || "";
      my $key = $username . "|" . $password;
  
      if (!$clients{$key}) {
        $client = BZ::Client->new("url" => $url);
        
        # Only attempt a login if we have an actual username and password
        if ($password) {
          $client->user($username);
          $client->password($password);
      
          eval {
            $c->stats->profile(begin => "Bugzilla XML-RPC request: login");
            $client->login();
            $c->stats->profile(end => "Bugzilla XML-RPC request: login");
          };
          if ($@) {
            $result = xmlrpc_error2error($@);
          }
        }
        
        if (!$result->{'error'}) {
          $clients{$key} = $client;
        }
      }
      
      $client = $clients{$key};
    }
  }

  if (!$result->{'error'}) {
    # Logging
    if (!$client->logger()) {
      if ($c->config->{'Log::Handler'}->{'filename'} &&
          $c->config->{'log_bzclient'}) 
      {
        my $logger = sub {
          # All logging in BZ::Client appears to be debug logging, so we don't
          # need to check $_[0].
          $c->log->info($_[1]);
        };
        
        $client->logger($logger);      
      }
      
      # Logging of actual contents of requests
      if ($c->config->{'log_xmlrpc'}) {
        $client->logDirectory("/tmp");
      }      
    }
    
    $c->log->info(stringify_req($c->req) . " => $method: " 
                                                         . jsonify($payload));
    
    eval {
      $c->stats->profile(begin => "Bugzilla XML-RPC request: $method");
      $result = $client->_api_call($method, $payload);
      $c->stats->profile(end => "Bugzilla XML-RPC request: $method");
    };
    if ($@) {
      # On Bugzilla 3.2, the add_comment call returns undef on success. :-(
      # This is the only way of detecting this; see BZ/Client.pm line 147.
      if ($method eq "Bug.add_comment" &&
          $@->{'message'} eq "Empty response from server." &&
          $c->config->{'bugzilla_version'} eq "3.2") {
        # Reset result
        $result = {};
      }
      else {
        $result = xmlrpc_error2error($@);
      }
    }    
  }
  return $result;  
}

# $ua can be undef
sub do_auth {
  my ($c, $url, $ua) = @_;
  
  # Always have at least an empty jar in case Bugzilla logs us in with URL
  # parameters and then 302 redirects us.
  $ua->cookie_jar({}) if $ua;
  
  my $cookie_jar = undef;
      
  if ($c->req->params->{'password'}) {
    # Username/password auth
    $url->query_param('Bugzilla_login', $c->req->params->{'username'});
    $url->query_param('Bugzilla_password', $c->req->params->{'password'});
  }
  elsif ($c->req->params->{'cookie'}) {
    # Put cookie in $ua
    my $server = new URI($c->config->{'bugzilla_url'});
    
    my $host = $server->host;
    
    # Hack for local testing (grr)
    $host .= ".local" if $host eq "localhost";
    
    $cookie_jar = HTTP::Cookies->new();
    $cookie_jar->set_cookie(0, 
                            "Bugzilla_logincookie", 
                            $c->req->params->{'cookie'}, 
                            $server->path, 
                            $host, 
                            $server->port);
    $cookie_jar->set_cookie(0, 
                            "Bugzilla_login", 
                            $c->req->params->{'userid'}, 
                            $server->path, 
                            $host, 
                            $server->port);
                            
    # We insert cookies into a LWP::UserAgent if given, but also return the
    # jar in case the object we want the cookies for isn't an instance of
    # that class.
    if ($ua) {
      $ua->cookie_jar($cookie_jar);
    }
  }
  
  return $cookie_jar;
}

1;
